//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormLog.h"
#include "ScopedLock.h"
#include "Mutex.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmLog *frmLog;
//---------------------------------------------------------------------------
__fastcall TfrmLog::TfrmLog(TComponent* Owner)
	: TForm(Owner)
{
	callbackClose = NULL;
	bUpdated = false;
	iBufSize = 1024;
}
//---------------------------------------------------------------------------

void TfrmLog::OnLog(char *buf)
{
	ScopedLock<Mutex> lock(mutex);
	que.push_back(buf);
	if (que.size() > iBufSize)
		que.pop_front();
    bUpdated = true;
}

void __fastcall TfrmLog::tmrUpdateTimer(TObject *Sender)
{
	if (!Visible || !bUpdated)
		return;
	ScopedLock<Mutex> lock(mutex);
	ListView->Items->Count = que.size();
	ListView->Invalidate();
    bUpdated = false;
}
//---------------------------------------------------------------------------

void __fastcall TfrmLog::ListViewData(TObject *Sender, TListItem *Item)
{
	ScopedLock<Mutex> lock(mutex);
	Item->Caption = que[Item->Index].c_str();
}
//---------------------------------------------------------------------------

void __fastcall TfrmLog::FormClose(TObject *Sender, TCloseAction &Action)
{
	if (callbackClose)
		callbackClose();
}
//---------------------------------------------------------------------------


void __fastcall TfrmLog::btnClearClick(TObject *Sender)
{
    ScopedLock<Mutex> lock(mutex);
    que.clear();
	ListView->Items->Count = que.size();
	ListView->Invalidate();
}
//---------------------------------------------------------------------------

void TfrmLog::SetBufferSize(int size)
{
    if (size < 0) size = 1024;
	iBufSize = size;
}

